@@ -69,7 +69,6 @@ from deployv.helpers import container
 from deployv.helpers.utils import get_error_message, decode
 from deployv.base import errors
 from retry_decorator import retry
-import psutil
 
 
 logger = logging.getLogger(__name__)  # pylint: disable=C0103
@@ -217,7 +216,7 @@ class DockerV(object):
         volume_binds = container.generate_binds(
             self.__config.get("volumes"), working_folder)
         volumes = container.generate_volumes(self.__config.get("volumes"))
-        self.setting_default_ports()
+        self.set_default_ports()
         ports = container.generate_port_lists(self.__config.get("ports"))
         port_bindings = container.generate_port_bindings(self.config.get('ports'))
         container_config = {
@@ -235,6 +234,7 @@ class DockerV(object):
             "binds": volume_binds,
             "port_bindings": port_bindings,
             "mem_limit": self.config.get("mem_limit", 0),
+            "memswap_limit": self.config.get("mem_limit", 0),
             "restart_policy": {
                 "MaximumRetryCount": 0,
                 "Name": "unless-stopped"
@@ -267,7 +267,10 @@ class DockerV(object):
             if 'port is already allocated' in get_error_message(error) \
                     or 'address already in use' in get_error_message(error):
                 logger.warning('Port already allocated')
-                raise errors.ErrorPort(error.explanation)
+                raise errors.ErrorPort(
+                    'At least one of the ports used by this container are being used by another'
+                    ' process, manual intervention is required. Error message:\n{msg}'
+                    .format(msg=error.explanation))
         return self.basic_info
 
     def remove_container(self, container_name=False):
@@ -319,8 +322,11 @@ class DockerV(object):
         :returns: Container information.
         :rtype: dict
         """
-        inspected = self.inspect()
         res = {}
+        try:
+            inspected = self.inspect()
+        except errors.NoSuchContainer:
+            return res
         if inspected.get('Id') == self.docker_id:
             res.update(
                 {
@@ -683,18 +689,85 @@ class DockerV(object):
         logger.error('Failed to restart the container, state: %s', container_obj.status)
         return False
 
-    def setting_default_ports(self):
-        """Setting ports by default. Searching all ports in use to filter with the ports
-        configured in the range.
+    def set_default_ports(self):
+        """Sets default port mappings if the ones specified in the json
+        contain falsy values so we can have control over the ports assigned
+        to the instances. It also checks if the ports specified for a
+        develop instance are free, it sets new ports if they are not.
         """
         deploy_cfg = self._InstanceV__full_config.deployer
+        instance_ports = self.config.get('ports')
+        specified_ports = set(self._parse_ports(instance_ports))
+        used_ports = set(self._get_used_ports())
         port_start = int(deploy_cfg.get('docker_start_port', 30000))
         port_end = int(deploy_cfg.get('docker_end_port', 40000))
-        ports = [key for key, val in self.__config.get('ports').items() if not val]
-        netstat = psutil.net_connections()
-        used_ports = [
-            nt.laddr.port for nt in netstat if nt.status in ['LISTEN', 'ESTABLISHED']]
-        available_ports = [p for p in range(port_start, port_end) if p not in used_ports]
-        if not ports or not available_ports:
+        available_specified_ports = list(specified_ports - used_ports)
+        available_default_ports = [
+            p for p in range(port_start, port_end) if
+            p not in used_ports
+        ]
+        available_ports = available_default_ports + available_specified_ports
+        if not available_ports:
+            raise errors.ErrorPort('There are no available ports in the range'
+                                   ' {start} - {end} to deploy this instance'
+                                   .format(start=port_start, end=port_end))
+        instance_type = self._InstanceV__full_config.instance_config.get(
+            'instance_type')
+        ports_to_map = [
+            port for port, mapping in instance_ports.items() if not mapping
+        ]
+        if instance_type == 'develop':
+            ports_to_map += [
+                port for port, mapping in
+                instance_ports.items() if mapping not in available_ports
+            ]
+        if not ports_to_map:
+            # There are no ports to map, try to use the ones specified
             return
-        self.__config.get('ports').update(dict(zip(ports, available_ports)))
+        instance_ports.update(dict(zip(ports_to_map, available_ports)))
+        return
+
+    def _get_used_ports(self):
+        """Method that checks the ports of all the containers in the server
+        in order to list all the ports used by instances in order to decide
+        if the current container's ports should be reassigned. This only checks
+        for the ports used by containers because the container's ports are
+        reassgined only if the ports are used by another container, if they are
+        used by another process (or zombie process), an error will be raised to
+        alert the users.
+        """
+        containers = self.cli2.containers.list()
+        used_ports = set()
+        for container_obj in containers:
+            container_info = self.inspect(container_obj.id)
+            ports = self._parse_ports(container.get_ports_dict(container_info))
+            used_ports.update(ports)
+        return list(used_ports)
+
+    def _parse_ports(self, ports):
+        """ Receives a dictionary with the port mappings between the container
+        and the host and parses them to return a list of the host ports used by
+        the containers.
+
+        :param ports: dictionary of port bindings
+        :return: List of integers representing the host ports used by the host
+        """
+        parse_ports = {
+            int: lambda x: x,
+            str: lambda x: int(x.split(':')[-1]),
+        }
+        parse_ports[list] = lambda x: [
+            parse_ports[type(port)](port) for port in x if
+            port and isinstance(port, (int, str))
+        ]
+        res = set()
+        for port in ports.values():
+            parse_func = parse_ports.get(type(port))
+            if not parse_func:
+                continue
+            parsed_ports = parse_func(port)
+            if not parsed_ports:
+                continue
+            res.update(parsed_ports if
+                       isinstance(parsed_ports, list) else [parsed_ports])
+        return list(res)
