@@ -1,10 +1,8 @@
 # coding: utf-8
 
 import sys
-import time
 import click
 from pkg_resources import working_set
-import simplejson as json
 import deployv
 from deployv.base import commandv
 from deployv.helpers import utils, build_helper, configuration_helper, json_helper
@@ -106,9 +104,13 @@ cli = DeployvCLI()  # pylint: disable=C0103
 @click.option("-d", "--database", help="Database name to be used",
               default=False, required=False)
 @click.option("--without-demo", is_flag=True, default=False,
-              help="If true, no database will be loaded demo data"
+              help="If True, no database will be loaded demo data"
                    " inside the container. Default False")
-def create(ctx, config_file, backup_src, external, key_file, without_db, database, without_demo):
+@click.option("--skip-push", is_flag=True, default=False,
+              help="If True, the images built during the create will not be pushed"
+                   " to quay.io. Default: False")
+def create(ctx, config_file, backup_src, external, key_file,
+           without_db, database, without_demo, skip_push):
     """Creates an instance then loads a backup from backup_dir folder.
     """
     config_obj = ctx.parent.command.config_parser
@@ -120,6 +122,7 @@ def create(ctx, config_file, backup_src, external, key_file, without_db, databas
         sys.exit(1)
     config_dict.get("container_config").get("env_vars").update({"without_demo": without_demo})
     config_dict.get("instance").get("config").update({"db_name": database} if database else {})
+    config_dict.get('instance').update({'restore_db': not without_db})
     if not config_dict.get('instance').get('config').get('db_owner'):
         config_dict.get('instance').get('config').update({
             'db_owner': config_obj.postgres.get('db_user')})
@@ -127,26 +130,14 @@ def create(ctx, config_file, backup_src, external, key_file, without_db, databas
         config_dict.get('instance').get('config').update({
             'db_owner_passwd': config_obj.postgres.get('db_password')})
     config_dict.get('group_config').update({'backup_src': backup_src} if backup_src else {})
-    config_dict.get('group_config').update({'command': 'deployvcmd.commandv.create'})
+    config_dict.get('group_config').update({
+        'command': 'deployvcmd.commandv.create',
+        'create_container': True,
+        'skip_push': skip_push
+    })
     config_obj.generate_configuration(config_dict)
     command = commandv.CommandV(config_obj)
-    res_create = command.create()
-    if not res_create.get('result'):
-        logger.error('Could not create the instance, some error raised: %s',
-                     res_create.get('error'))
-        sys.exit(1)
-    if without_db:
-        sys.exit(0)
-    time.sleep(10)
-    res_restore = command.restore()
-    if res_restore.get('result', False):
-        logger.info('Done')
-    else:
-        logger.warn(
-            'Could not restore any backup to the desired instance: %s',
-            res_restore.get('error'))
-        sys.exit(1)
-    sys.exit(0)
+    command.create_trigger()
 
 
 @cli.command()
@@ -161,7 +152,7 @@ def create(ctx, config_file, backup_src, external, key_file, without_db, databas
               default=False, required=False)
 @click.option("--tmp-dir", help="Temp dir to use (optional)",
               default=False, required=False)
-@click.option("-c", "--cformat", help="Compression format to be used (tar.bz2 or tar.gz)",
+@click.option("-c", "--cformat", help="Compression format to be used (tar.bz2, tar.gz or tar)",
               default='bz2', required=False)
 @click.option("-d", "--database", help="Database name",
               default=False, required=True)
@@ -195,12 +186,7 @@ def backupdb(ctx, config_file, backup_dir, reason, tmp_dir, database, cformat,
     config_dict['instance']['config'].update({'db_name': database} if database else {})
     config_obj.generate_configuration(config_dict)
     command = commandv.CommandV(config_obj)
-    res = command.backup()
-    if res.get('result', False):
-        logger.info('Backup generated: %s', res.get('result'))
-        logger.info('Done')
-    else:
-        logger.error('Could not generate the backup: %s', res.get('error'))
+    command.backup_trigger()
 
 
 @cli.command()
@@ -259,6 +245,7 @@ def restore(ctx, config_file, backup_src, database, container, admin_pass,
                 'Could not get the name of the database, '
                 'please use the -x or -d parameter')
             sys.exit(1)
+            config_dict['instance'].update({'restore_db': True})
     config_dict['instance']['config'].update({'db_name': database} if database else {})
     config_dict.get('group_config').update({'backup_src': backup_src} if backup_src else {})
     config_dict.get('group_config').update({'jobs': jobs} if jobs else {})
@@ -266,17 +253,7 @@ def restore(ctx, config_file, backup_src, database, container, admin_pass,
     config_dict.get('group_config').update({'command': 'deployvcmd.commandv.restore'})
     config_obj.generate_configuration(config_dict)
     command = commandv.CommandV(config_obj)
-    res = command.restore()
-    if res.get('result', False):
-        message = "Backup {backup_name} restored in database {dbname}".\
-            format(backup_name=res.get('result').get('backup'),
-                   dbname=res.get('result').get('database_name'))
-        logger.info(message)
-        logger.info('Done')
-    else:
-        logger.error('Could not restore the backup: %s', res.get('error'))
-        sys.exit(1)
-    sys.exit(0)
+    command.restore_trigger()
 
 
 @cli.command()
@@ -296,20 +273,7 @@ def update(ctx, config_file, database):
     config_dict.get('group_config').update({'command': 'deployvcmd.commandv.updatedb'})
     config_obj.generate_configuration(config_dict)
     command = commandv.CommandV(config_obj)
-    res_create = command.create()
-    if res_create.get('result', False):
-        res_update = command.updatedb()
-        if res_update.get('error', False):
-            logger.error(
-                'An error was detected during update process: %s', res_update.get('error'))
-            sys.exit(1)
-        else:
-            logger.info('Update done')
-            sys.exit(0)
-    else:
-        logger.error('the instance was not created: %s',
-                     res_create.get('error'))
-        sys.exit(1)
+    command.update_trigger()
 
 
 @cli.command()
@@ -335,11 +299,14 @@ def deactivate(ctx, container, database, admin_password, nginx_url):
                 'admin': admin_password,
                 'db_name': database
             }
+        },
+        'group_config': {
+            'deactivating_database': True
         }
     }
     config_obj.generate_configuration(config_dict)
     command = commandv.CommandV(config_obj)
-    command.deactivate()
+    command.deactivate_trigger()
 
 
 @cli.command()
@@ -378,20 +345,12 @@ def deactivate_backup(ctx, prefix, store_path, backup_src, admin_password,
     default_config = utils.load_default_config()
     default_config['group_config'].update({
         'jobs': jobs, 'no_fs': no_fs, 'store_path': store_path,
-        'backup_src': backup_src, 'cformat': cformat, 'prefix': prefix})
+        'remove': remove, 'deactivating_backup': True, 'backup_src': backup_src,
+        'cformat': cformat, 'prefix': prefix, 'upload': upload})
     default_config['instance']['config'].update(db_config)
     config_obj.generate_configuration(default_config)
     command = commandv.CommandV(config_obj)
-    res = command.deactivate_backup()
-    if not res:
-        sys.exit(1)
-    logger.info('Deactivated backup stored in %s', res)
-    if upload:
-        utils.upload_file(res, upload)
-        logger.info('File uploded to %s', upload)
-    if remove:
-        os.remove(res)
-    logger.info('Done')
+    command.deactivate_backup_trigger()
 
 
 @cli.command()
@@ -421,8 +380,11 @@ def deactivate_backup(ctx, prefix, store_path, backup_src, admin_password,
                     " in the format namespace/reponame#branch. Example: odoo/odoo#8.0"))
 @click.option("-T", "--tag", default=False,
               help="Custom name for the new image, if not specified a name will be generated")
+@click.option("--skip-push", is_flag=True, default=False,
+              help="If True, the images built during the create will not be pushed"
+                   " to quay.io. Default: False")
 def build(ctx, config_file, repo, branch, image_name, working_folder,
-          version, force, odoo_repo, tag):
+          version, force, odoo_repo, tag, skip_push):
     config_obj = ctx.parent.command.config_parser
     config_dict = utils.merge_config(config_file or {})
     if not config_dict:
@@ -459,15 +421,13 @@ def build(ctx, config_file, repo, branch, image_name, working_folder,
             'db_owner_passwd': config_obj.postgres.get('db_password')
         })
     config_dict.get('container_config').update({'build_image': True})
-    config_dict.get('group_config').update({'working_folder': working_folder, 'version': version,
-                                            'force': force, 'tag': tag})
+    config_dict.get('group_config').update({
+        'working_folder': working_folder, 'version': version,
+        'force': force, 'tag': tag, 'skip_push': skip_push
+    })
     config_obj.generate_configuration(config_dict)
-    res = build_helper.build_image(config_obj)
-    if not res[0]:
-        logger.error(res[1].get('error'))
-        sys.exit(1)
-    logger.info('Image built: %s', res[1].get('result').get('image_name'))
-    sys.exit(0)
+    command = commandv.CommandV(config_obj)
+    command.build_image_trigger()
 
 
 @cli.command(short_help="Creates the default configuration.")
@@ -570,9 +530,4 @@ def update_production(ctx, config_file, container, db_name, db_user, db_password
         sys.exit(1)
     config_obj.generate_configuration(config_dict, load_defaults=False)
     command = commandv.CommandV(config_obj)
-    res = command.update_production()
-    if res.get('error'):
-        logger.error(res.get('error'))
-        sys.exit(1)
-    logger.info('Update result: %s', json.dumps(res.get('result'), indent=2))
-    sys.exit(0)
+    command.update_prod_trigger()
