@@ -5,7 +5,7 @@ from deployv.instance import ODOO_BINARY, SAAS_VERSIONS
 import docker
 from docker import APIClient as Client
 from jinja2 import Environment, FileSystemLoader
-from os import path
+from os import path, getenv
 from tempfile import mkdtemp
 import shutil
 import logging
@@ -102,15 +102,16 @@ class BuildImage(object):
         with open(path.join(self._files_folder, 'odoo_service.conf'), 'w') as text_file:
             text_file.write(service_file)
 
-    def _render_postgres_service(self):
+    def _render_postgres_service(self, psql_version=False):
         """ Render the supervisor configuration file for postgres depending on the `full_stack`
         parameter.
 
         :return: None
         """
+        psql_version = psql_version or '9.6'
         env = Environment(loader=FileSystemLoader(self._templates_path))
         template = env.get_template('postgres_service.conf.jinja')
-        service_file = template.render({'full_stack': self.full_stack_image})
+        service_file = template.render({'psql_version': psql_version})
         _logger.debug('Rendering supervisor config for Postgresql')
         with open(path.join(self._files_folder, 'postgres_service.conf'), 'w') as text_file:
             text_file.write(service_file)
@@ -162,7 +163,6 @@ class BuildImage(object):
                           ' use --force to remove it')
             raise
         self._render_odoo_service()
-        self._render_postgres_service()
         self._render_entrypoint_image()
         self._render_create_user_db()
         instance_path = path.join(self._files_folder, 'instance')
@@ -291,7 +291,81 @@ class BuildImage(object):
                 msg = [val for val in ansi_escape.sub('', msg).split("\n") if val]
                 raise BuildError(msg[len(msg)-1].strip())
         image_sha = self.client.images(name=self.new_image_name, quiet=True)
-        res = image_sha and image_sha[0].decode().split(':')[1][:10]
+        res = image_sha and utils.decode(image_sha[0]).split(':')[1][:10]
+        return res
+
+    def _find_file(self, file_path, file_name):
+        """Searches for a file with the provided name in the provided path,
+        reads it, and returns its contents.
+
+        :param file_path: Path where the search will be done
+        :param file_name: Name of the file to find
+        :return: The contents of the file or False if the file was not found
+        """
+        shell = spur.LocalShell()
+        file_path = shell.run(
+            ['find', file_path, '-name', file_name]
+        )
+        file_path = file_path.output.strip()
+        if not path.exists(file_path):
+            return False
+        with open(file_path, 'r') as obj:
+            file_contents = obj.read()
+        return file_contents
+
+    def _get_variables(self):
+        """Searches for a file named `variables` among the cloned repositories
+        and parses it to make the variables within it usable during the build.
+
+        :return: List of environment variables (strings) in the format `var=val`
+        :rtype: list
+        """
+        variables = self._find_file(self.temp_working_folder, 'variables.sh')
+        if not variables:
+            return []
+        variables = variables.replace('export', '').strip().split('\n')
+        return variables
+
+    def _apply_patches(self, instance_path):
+        """Searches for a file named `patches.txt` and applies the git patches
+        specified in it directly in the specified repositories.
+        """
+        res = {'result': []}
+        shell = spur.LocalShell()
+        patches = self._find_file(self.temp_working_folder, 'patches.txt')
+        if not patches:
+            return res
+        patches = patches.strip().split('\n')
+        headers = [
+            '--header=Authorization: token %s' % getenv('GITHUB_TOKEN'),
+            '--header=Accept: application/vnd.github.VERSION.patch'
+        ]
+        for patch in patches:
+            patch_parts = patch.split(' ', 1)
+            repo = patch_parts[0]
+            url = patch_parts[1]
+            file_path = path.join(self._config.temp_folder, 'deployv.patch')
+            cmd = ['wget', url, '-O', file_path]
+            if 'api.github.com' in url:
+                _logger.debug('A patch from a private repo will be applied: %s', url)
+                cmd += headers
+            try:
+                shell.run(cmd)
+            except spur.results.RunProcessError as error_obj:
+                error = ('An error occurred while downloading the patch %s:\n%s' %
+                         (url, error_obj.stderr_output))
+                _logger.error(error)
+                return {'error': error}
+            repo_path = path.join(instance_path, repo)
+            try:
+                shell.run(['patch', '-f', '-p1', '-i', file_path, '-d', repo_path])
+            except spur.results.RunProcessError as error_obj:
+                error = ('An error occurred while applying the patch %s:\n%s' %
+                         (url, error_obj.stderr_output))
+                _logger.error(error)
+                return {'error': error}
+            shell.run(['rm', file_path])
+            res.get('result').append(url)
         return res
 
     def build(self):
@@ -320,13 +394,25 @@ class BuildImage(object):
                 utils.clean_files([self.temp_working_folder])
                 res.update({'error': utils.get_error_message(error)})
                 return res
+            variables = self._get_variables()
+            psql_variable = [variable for variable in variables if 'PSQL_VERSION' in variable]
+            psql_version = psql_variable and psql_variable[0].split('=')[1]
+            if not psql_version:
+                psql_version = '9.6'
+                variables.append('PSQL_VERSION=9.6')
             self._generate_apt_requirements()
+            self._render_postgres_service(psql_version)
+            patches_res = self._apply_patches(paths['instance'])
+            if patches_res.get('error'):
+                return patches_res
+            _logger.info('Patches applied: %s', '\n'.join(patches_res.get('result')))
             values = {
                 'version': self.version,
+                'variables': variables,
                 'base_image_name': self.base_image_name,
+                'full_stack': self.full_stack_image,
                 'db_user': self._config.instance_config.get('config').get('db_user'),
                 'db_password': self._config.instance_config.get('config').get('db_password'),
-                'full_stack': self.full_stack_image
             }
             template = 'build_dockerfile.jinja'
             res = self._start_build(template, values)
@@ -409,9 +495,9 @@ def build_image(config):
         version = config.instance_config.get('version') or utils.version_cid(
             config.instance_config.get('customer_id'))
         if version is None:
-            return (False, {'error': ('You need to specify the customer id with the correct format'
-                                      ' or use the version parameter. Example: customer80 or '
-                                      'customersaas-14')})
+            return {'error': ('You need to specify the customer id with the correct format'
+                              ' or use the version parameter. Example: customer80 or'
+                              ' customersaas-14')}
         config.version = version
     _logger.debug('Starting the build image process with:')
     _logger.debug(json.dumps(config._deploy_config, sort_keys=True, indent=2))
@@ -419,11 +505,8 @@ def build_image(config):
     config.working_folder = config.working_folder or config.temp_folder
     if not config.container_config.get('build_image') and not full_stack:
         build_res.update({'error': 'You must use the build_image and/or full_stack parameters'})
-        return (False, build_res)
+        return build_res
     _logger.debug('Building single image')
     build_class = BuildImage(config, full_stack_image=full_stack)
     build_res = build_class.build()
-    if build_res.get("error"):
-        return (False, build_res)
-    config.container_config.update({'image_name': build_res.get('result').get('image_name')})
-    return (True, build_res)
+    return build_res
