@@ -36,7 +36,7 @@ class DeployvConfig(object):
             'deactivate': ['_deploy_config'],
             'restore': ['_deploy_config'],
             'backup': ['_deploy_config'],
-            'deactivate_backup': ['_deploy_config', 'temp_folder', 'backup_src', 'prefix'],
+            'deactivate_backup': ['_deploy_config', 'temp_folder', 'prefix'],
             'updatedb': ['_deploy_config'],
             'update_production': ['backup_src', 'compressed_backup_size',
                                   'decompressed_backup_size', 'temp_folder'],
@@ -125,7 +125,8 @@ class DeployvConfig(object):
         :rtype: ConfigParser
         """
         config = self.worker_config or ConfigParser()
-        default_config_file = 'deployv/config/deployv.conf'
+        default_config_file = os.path.join(os.path.dirname(__file__),
+                                           '../config/deployv.conf')
         config_files = [default_config_file]
         main_dirs = ['/etc/deployv', os.path.expanduser('~/.config/deployv')]
         for main_dir in main_dirs:
@@ -249,11 +250,11 @@ class DeployvConfig(object):
 
     def setup_config_dirs(self):
         _logger.info("Writing config files")
-        default_config_file = 'deployv/config/deployv.conf'
+        default_config_file = os.path.join(os.path.dirname(__file__),
+                                           '../config/deployv.conf')
         for main_dir in ['/etc/deployv', os.path.expanduser('~/.config/deployv')]:
             config_file_name = os.path.join(main_dir, 'deployv.conf')
             error_msg = "Couldn't write config file: '%s' (skipped)"
-            ok_msg = "Created config file: '%s'"
             addon_dir = os.path.join(main_dir, 'conf.d')
             is_user_dir = os.path.expanduser('~/') in main_dir
             try:
@@ -271,10 +272,10 @@ class DeployvConfig(object):
             try:
                 with open(config_file_name, 'w+') as config_file:
                     config.write(config_file)
-            except (OSError, IOError) as error:
+            except (OSError, IOError):
                 _logger.warn(error_msg, config_file_name)
             else:
-                _logger.info(ok_msg, config_file_name)
+                _logger.info("Created config file: '%s'", config_file_name)
                 if is_user_dir:
                     self._fix_ownership(config_file_name)
         self.worker_config = self.parse_config()
