@@ -3,7 +3,7 @@ import zipfile
 
 
 class DecompressHelper(object):
-    valid_extensions = ('tar.gz', 'tar.bz2', 'zip')
+    valid_extensions = ('tar.gz', 'tar.bz2', 'zip', 'tar')
 
     def __init__(self, filename):
         self.fobject = self.get_decompress_object(filename)
@@ -13,6 +13,7 @@ class DecompressHelper(object):
         suport_method = {
             "tar.gz": lambda fname: tarfile.open(fname, mode='r:gz'),
             "tar.bz2": lambda fname: tarfile.open(fname, mode='r:bz2'),
+            "tar": lambda fname: tarfile.open(fname, mode='r:'),
             "zip": lambda fname: zipfile.ZipFile(fname, mode='r')
         }
         return suport_method
