@@ -46,6 +46,8 @@ class SenderMessage(object):
         if body:
             message_body.update(body)
         if msg:
+            log = logger.info if log_type == 'INFO' else logger.error
+            log(msg)
             message_body.update({'log': '{time} {type} {module}: {msg}'.format(
                 time=datetime.utcnow().isoformat(' '), type=log_type, module=self.__config.command,
                 msg=msg)
