@@ -22,9 +22,14 @@ from deployv.helpers import container, json_helper
 from deployv.helpers.decompress_helper import DecompressHelper
 from deployv import base
 from docker.errors import APIError
+from jinja2 import Template
+import socket
+import fcntl
+import struct
+import ipaddress
 
 _AVAILABLES = {
-    "production": ['update_production', 'post_process'],
+    "production": ['update_production', 'post_process', 'restart_instance', 'restart_container'],
     "test": [],
     "updates": [],
     "develop": [],
@@ -251,6 +256,8 @@ def get_decompress_object(file_name):
         fobject, modestr = tarfile.open, 'r:gz'
     elif file_name.endswith('tar.bz2'):
         fobject, modestr = tarfile.open, 'r:bz2'
+    elif file_name.endswith('tar'):
+        fobject, modestr = tarfile.open, 'r:'
     else:
         raise RuntimeError('Unknown file format "{}"'.format(file_name))
     return fobject, modestr
@@ -382,16 +389,22 @@ def compress_files(name, files, dest_folder=None, cformat='bz2'):
     """
     if not dest_folder:
         dest_folder = '.'
-    if cformat not in ['bz2', 'gz']:
+    if cformat not in ['bz2', 'gz', 'tar']:
         raise RuntimeError('Unknown file format "{}"'.format(cformat))
+    ext = ''
     if cformat == 'gz':
         fobject, modestr = tarfile.open, 'w:gz'
+        ext = 'tar.gz'
+    elif cformat == 'tar':
+        fobject, modestr = tarfile.open, 'w:'
+        ext = 'tar'
     elif cformat == 'bz2':
         fobject, modestr = tarfile.open, 'w:bz2'
+        ext = 'tar.bz2'
     logger.debug("Generating compressed file: %s in %s folder",
                  name, dest_folder)
 
-    bkp_name = '{0}.tar.{1}'.format(name, cformat)
+    bkp_name = '{0}.{1}'.format(name, ext)
     full_tmp_name = os.path.join(
         dest_folder,
         '._{}'.format(bkp_name)
@@ -711,6 +724,19 @@ def get_certificates_pac():
     return certificates
 
 
+def render_template(template, values=False):
+    """Renders the provided jinja template with the provided values and
+    returns the rendered template.
+    """
+    values = values or {}
+    template_path = os.path.join(os.path.dirname(__file__), '..',
+                                 'templates', template)
+    with open(template_path, 'r') as obj:
+        json_template = obj.read()
+    res = json_helper.load_json(Template(json_template).render(values))
+    return res
+
+
 def is_base64_encode(string_encode):
     """verifies that the string is a base64 encoding
     """
@@ -751,3 +777,26 @@ def get_size(path):
     # so the methods that parse the sizes can accept it
     res = res.output.split()[0].decode() + 'B'
     return res
+
+
+def get_images_path():
+    """Gets the path where the docker images are stored.
+    """
+    shell = spur.LocalShell()
+    res = shell.run(shlex.split('docker info -f "{{.DockerRootDir}}"')).output
+    return res.strip('\n')
+
+
+def get_ip_address(ifname):
+    tmp_sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+    return socket.inet_ntoa(fcntl.ioctl(
+        tmp_sock.fileno(),
+        0x8915,
+        struct.pack('256s', ifname[:15])
+    )[20:24])
+
+
+def is_docker_ip(ip_addr):
+    """Recevives an ip and checks if it is in the docker network range"""
+    docker0 = re.sub(r"\.\d{1,3}$", ".0/24", get_ip_address('docker0'))
+    return ipaddress.ip_address(unicode(ip_addr)) in ipaddress.ip_network(unicode(docker0))
