@@ -91,9 +91,8 @@ import simplejson as json
 from passlib.context import CryptContext
 from deployv.base.dockerv import DockerV
 from deployv.base.postgresv import PostgresShell, PostgresConnector
-from deployv.base import postgresv
-from deployv.helpers import utils, container, build_helper, database_helper, json_helper
-from deployv.base import errors
+from deployv.base import postgresv, errors
+from deployv.helpers import utils, container, database_helper, json_helper
 from deployv.instance import ODOO_BINARY, SAAS_VERSIONS
 from datetime import datetime
 from jinja2 import Template
@@ -151,6 +150,7 @@ class InstanceV(DockerV):
                 'db_port': 5432,
             })
         self.pull_image()
+        self.start_postgres_container()
         logger.info('Starting Odoo container %s',
                     self.config.get('container_name'))
         self.config.update({'container_hostname': container.generate_hostname(self.__full_config)})
@@ -239,7 +239,9 @@ class InstanceV(DockerV):
         with open(authorized_keys_tar_path, 'r') as tar_data:
             self.cli.put_archive(self.config.get('container_name'),
                                  '/home/odoo/.ssh/', tar_data.read())
+            self.exec_cmd('chown odoo:odoo /home/odoo/.ssh/authorized_keys')
             self.exec_cmd('cp /home/odoo/.ssh/authorized_keys /root/.ssh/')
+            self.exec_cmd('chown root:root /root/.ssh/authorized_keys')
         utils.clean_files([authorized_keys_tar_path, authorized_keys_path])
 
     @property
@@ -560,18 +562,17 @@ class InstanceV(DockerV):
         getaddons_path = path.join(path.dirname(path.abspath(__file__)), '..',
                                    'templates', 'files', 'getaddons.py')
         shutil.copyfile(getaddons_path, path.join(self.temp_folder, 'getaddons.py'))
-        self.move_file('helpers-requirements.txt')
         self.move_file('clone_oca_dependencies.py')
         with open(branches_filename, 'w') as fout:
             json.dump(self.__full_config.instance_config.get('repositories'),
                       fout, indent=4, ensure_ascii=False,
                       separators=(',', ':'))
-        bash_lines = ['pip install -r /tmp/helpers-requirements.txt -q']
-        if not self.config.get("build_image"):
-            bash_lines += [
-                'su {user} -c "branchesv load -p {home}/instance -f /tmp/branches.json"']
-        for line in bash_lines:
-            res = self.exec_cmd(line.format(home=odoo_home, user=odoo_user))
+        if not self.config.get("build_image") or self.__full_config.updating_instance:
+            # Backwards compatibility, install branchesv in case the image
+            # used was not built with deployv or was built before the recent changes
+            self.exec_cmd('pip install branchesv')
+            cmd = 'su {user} -c "branchesv load -p {home}/instance -f /tmp/branches.json"'
+            res = self.exec_cmd(cmd.format(home=odoo_home, user=odoo_user))
             logger.debug(res)
             regex = re.compile(r'Cloning repo (.*?) - branch (.*)')
             matches = regex.finditer(res)
@@ -583,11 +584,12 @@ class InstanceV(DockerV):
                     branch = match.group(2)
                     logger.debug('Gotten repo: %s', repo)
                     logger.debug('Gotten branch: %s', branch)
-            error_regex = re.compile(r'[fatal|ERROR]: (.*)')
+            error_regex = re.compile(r'(fatal|ERROR): (?P<msg>.*)')
             fatal = error_regex.search(res)
             if fatal:
-                logger.error(fatal.group(1))
-                success = (False, fatal.group(1))
+                error = fatal.groupdict().get('msg')
+                logger.error(error)
+                success = (False, error)
         if success[0]:
             addons_path = "{home}/instance/extra_addons".format(home=odoo_home)
             self.get_oca_dependencies()
@@ -622,7 +624,13 @@ class InstanceV(DockerV):
         """
         if not self.__full_config.container_config.get('postgres_container'):
             return {}
-        psql_version = self.__full_config.container_config.get('postgres_version', '9.5')
+        try:
+            image_data = self.inspect_image(self.config.get('image_name'))
+            vars_list = image_data.get('Config').get('Env')
+        except errors.NoSuchImage:
+            vars_list = []
+        image_vars = container.parse_env_vars(vars_list)
+        psql_version = image_vars.get('PSQL_VERSION', '9.6')
         psql_image = "postgres:{version}".format(version=psql_version)
         config = self.config.copy()
         self.config.update({"image_name": psql_image})
@@ -646,6 +654,9 @@ class InstanceV(DockerV):
         ipaddress = self.inspect().get('NetworkSettings').get('IPAddress')
         self.config = config
         self.config.get('env_vars').update({'db_host': ipaddress})
+        # Don't expose the 5432 port in the odoo container
+        # if a postgres container was created
+        self.config.get('ports').pop('5432', False)
         return info
 
     def get_oca_dependencies(self):
@@ -701,34 +712,34 @@ class InstanceV(DockerV):
         :return: dict with the needed configuration parameter
         """
 
-        env_vars = self.docker_env
-        logger.debug(json.dumps(env_vars, sort_keys=True, indent=2))
-        if env_vars is None:
-            logger.warn(
-                "No such container (or not started yet)")
-            instance_cfg = self.__full_config.instance_config.get("config")
-            logger.debug("Instance config dict from json: %s",
-                         str(instance_cfg))
-            res = {
-                'db_host': instance_cfg.get('db_host') or '127.0.0.1',
-                'db_port': instance_cfg.get('db_port') or 5432,
-                'db_user': instance_cfg.get('db_user') or 'odoo',
-                'db_password': instance_cfg.get('db_password') or 'odoo',
-            }
-        else:
-            db_host = env_vars.get('db_host') or '127.0.0.1'
-            db_port = env_vars.get('db_port') or 5432
-            if self.basic_info.get('ports').get('5432'):
-                db_port = self.basic_info.get('ports').get('5432')
-                if db_port and isinstance(db_port, list):
-                    db_port = db_port[0].split(':')[1]
-            res = {
-                'db_host': db_host,
-                'db_port': db_port,
-                'db_user': env_vars.get('db_user') or 'odoo',
-                'db_password': env_vars.get('db_password') or 'odoo',
-            }
+        env_vars = self.docker_env or {}
+        logger.debug(json.dumps(env_vars, sort_keys=True, indent=4))
+        instance_cfg = self.__full_config.instance_config.get("config")
+        # If the `full_stack` parameter is True, use the env_vars over the json
+        # config, if it is False, use the json over the env vars. This is to avoid
+        # problems with the full stack instances that change the db config in the
+        # env vars.
+        configs = {True: instance_cfg, False: env_vars}
+        main_cfg = configs.get(not self.config.get('full_stack', False))
+        optional_cfg = configs.get(self.config.get('full_stack', False))
+        logger.debug("Instance config dict from json: %s", str(instance_cfg))
+        res = {
+            'db_host': main_cfg.get('db_host') or optional_cfg.get('db_host') or '127.0.0.1',
+            'db_port': main_cfg.get('db_port') or optional_cfg.get('db_port') or 5432,
+            'db_user': main_cfg.get('db_user') or optional_cfg.get('db_user') or 'odoo',
+            'db_password': (main_cfg.get('db_password') or
+                            optional_cfg.get('db_password') or 'odoo'),
+        }
+        if self.basic_info.get('ports', {}).get('5432'):
+            res.update({'db_port': self.basic_info.get('ports').get('5432')})
+        if isinstance(res.get('db_port'), list):
+            db_port = res.get('db_port')[0].split(':')[1]
+            res.update({'db_port': db_port})
+        try:
             inspected = self.inspect()
+        except errors.NoSuchContainer:
+            inspected = {}
+        if inspected:
             volumes = inspected.get('Mounts')
             for volume in volumes:
                 if '.local/share/Odoo' in volume.get('Destination') and volume:
@@ -737,77 +748,61 @@ class InstanceV(DockerV):
             else:
                 logger.error(
                     'Datadir not found, wont be able to backup attachments')
-
             if not res.get('data_dir'):
                 logger.error(('The attachments directory was not mounted from the host,'
                               ' wont be able to backup attachments'))
             res.update({'odoo_container': self.docker_id})
-        regex = re.compile(r'172\.17\.\d+\.1$')
-        match = regex.match(res.get('db_host'))
-        if match or res.get('db_host') == 'localhost':
+        if utils.is_docker_ip(res.get('db_host')) or res.get('db_host') == 'localhost':
             res.update({'db_host': '127.0.0.1'})
         return res
 
     def restore_database(self, db_name=False, backup_dir=False):
-        res = {}
-        backup_size = False
+        res = {'result': {}}
+        db_name_config = self.__full_config.instance_config.get('config').get('db_name')
         if self.config.get("keep_db", False):
-            db_name = self.__full_config.instance_config.get('config').get('db_name')
-            return (True, {"result": {'database_name': db_name}})
-        customer_id = self.__full_config.instance_config.get('customer_id') or\
-            self.docker_env.get("customer")
+            return {"result": {'database_name': db_name_config}}
+        customer_id = (self.__full_config.instance_config.get('customer_id') or
+                       self.docker_env.get("customer"))
         use_template = self.__full_config.container_config.get('use_template')
-        db_owner = self.__full_config.instance_config.get('config').get('db_owner')
-        db_owner_password = self.__full_config.instance_config.get('config').get('db_owner_passwd')
+        owner = self.__full_config.instance_config.get('config').get('db_owner')
+        passwd = self.__full_config.instance_config.get('config').get('db_owner_passwd')
         jobs = self.__full_config.jobs or 5
         temp_folder = self.__full_config.temp_folder or '/tmp'
         db_config = self.db_config.copy()
-        db_config.update({'db_user': db_owner, 'db_password': db_owner_password})
+        db_config.update({'db_user': owner, 'db_password': passwd})
         pg_shell = PostgresShell(utils.odoo2postgres(db_config))
         working_dir = mkdtemp(prefix='deployv_', dir=temp_folder)
         helper_obj = database_helper.DatabaseHelper.get_helper(use_template)
         db_helper = helper_obj(utils.odoo2postgres(self.db_config))
-        if use_template:
-            search_res = db_helper.search_candidate(customer_id)
-            if not search_res[0]:
-                return search_res
-            candidate = search_res[1]
-            filestore_dir = candidate.replace('original_', '')
-            filestore = path.join(path.expanduser(self.config.get('backup_folder')),
-                                  filestore_dir, 'filestore')
-        else:
-            if not backup_dir:
-                backup_dir = self.__full_config.backup_src
-            search_res = db_helper.search_candidate(backup_dir, customer_id)
-            if not search_res[0]:
-                return search_res
-            try:
-                dest_dir = utils.decompress_files(search_res[1], working_dir)
-            except (EOFError, IOError) as error:
-                utils.clean_files(working_dir)
-                return (False, utils.get_error_message(error))
-            candidate = dest_dir
-            backup_size = utils.get_size(dest_dir)
-            filestore = path.join(candidate, 'filestore')
-        if not db_name:
-            if self.__full_config.instance_config.get('config').get('db_name'):
-                db_name = self.__full_config.instance_config.get('config').get('db_name')
-            else:
-                db_name = utils.generate_dbname(
-                    self.__full_config, path.basename(search_res[1]), self.__full_config.prefix)
+        backup_dir = backup_dir or self.__full_config.backup_src
+        res_search = db_helper.search_candidate(backup_dir, customer_id)
+        if not res_search[0]:
+            return {'error': res_search[1]}
+        res.get('result').update({'backup': path.basename(res_search[1])})
+        try:
+            candidate = utils.decompress_files(res_search[1], working_dir)
+        except (EOFError, IOError) as error:
+            utils.clean_files(working_dir)
+            res.update({'error': utils.get_error_message(error)})
+            return res
+        backup_size = utils.get_size(candidate)
+        res.get('result').update({'backup': path.basename(candidate),
+                                  'source': candidate, 'backup_size': backup_size})
+        filestore = path.join(candidate, 'filestore')
+        db_name = db_name or db_name_config or utils.generate_dbname(
+            self.__full_config, path.basename(res_search[1]), self.__full_config.prefix)
         self.stop_instance()
         db_name = db_name.lower()
-        create_res = db_helper.create_database(candidate, db_name,
-                                               db_owner, db_owner_password, jobs=jobs)
+        create_res = db_helper.create_database(candidate, db_name, owner, passwd, jobs=jobs)
         if not create_res[0]:
-            return create_res
+            res.update({'error': create_res[1]})
+            return res
         create_date = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
         self.restore_filestore(filestore, db_name)
-        res.update({"result": {'database_name': db_name, 'source': candidate,
-                               'create_date': create_date, 'backup_size': backup_size}})
+        res.get('result').update({'database_name': db_name, 'create_date': create_date})
         utils.clean_files(working_dir)
         pg_shell.create_extension_unaccent(db_name)
-        return (True, res)
+        return res
 
     def files2backup(self, database_name, tmp_dir):
         """ Generates the files that will be added to the backup, such as
@@ -954,6 +949,8 @@ class InstanceV(DockerV):
         """
         logger.debug('Getting properties')
         data_dir = self._get_fstore_from_config()
+        if not data_dir:
+            raise errors.FileNotFoundInContainer(self.docker_id, 'filestore')
         fs_name = path.join(data_dir, "filestore", db_name)
         volumes = self.config.get('volumes')
         working_folder = path.join(self.config.get("working_folder"),
@@ -967,9 +964,15 @@ class InstanceV(DockerV):
             newfs=path.join(data_dir, "filestore"))
         logger.debug('Executing commands "%s"', cmd)
         full_cmd = "bash -c '" + cmd + "'"
-        self.cli2.containers.run(
+        cont = self.cli2.containers.run(
             self.config.get('image_name'), full_cmd,
-            volumes=binded_volumes, detach=True, auto_remove=True)
+            volumes=binded_volumes, detach=True)
+        while cont.status != 'exited':
+            logger.debug(
+                'Waiting for the auxiliar container to finish, current state: %s', cont.status)
+            time.sleep(5)
+            cont.reload()
+        cont.remove()
 
     def _get_fstore_from_config(self):
         data_dir = None
@@ -980,13 +983,15 @@ class InstanceV(DockerV):
             res = self.cli.get_archive(container_name, odoo_config_file)
             logger.debug('Got from get_archive: %s', str(res))
         except errors.FileNotFoundInContainer:
-            logger.error("Odoo config file is not in the path '%s'", odoo_config_file)
-            return None
-        for line in utils.decode(res[0].data).split('\n'):
-            if line.strip().startswith("data_dir"):
-                data_dir = line.split("=")[1].strip()
-                break
-        return data_dir
+            raise errors.FileNotFoundInContainer(container_name, odoo_config_file)
+        # The first element of the result returned by `get_archive()` is now
+        # a generator that returns the contents of the file.
+        for content in res[0]:
+            lines = utils.decode(content).split('\n')
+            for line in lines:
+                if line.strip().startswith("data_dir"):
+                    data_dir = line.split("=")[1].strip()
+                    return data_dir
 
     def clone_db(self, database):
         """ Creates a new database copy of another database
@@ -1340,22 +1345,6 @@ class InstanceV(DockerV):
         res = self.install_packages(apt_list, pip_list)
         return res
 
-    def build_image(self):
-        """ Calls the build helper in order to build the image that
-        will be used to create the container, updates the configuration
-        with the name of the new image and removes all the repos in the config
-        if the build_image parameter is true so the repos in the image are not
-        overriden when the container is created.
-
-        :return: dict with the result of the build
-        """
-        build_res = build_helper.build_image(self.__full_config)
-        if build_res[0] and build_res[1].get('result'):
-            new_image = build_res[1].get('result').get('image_name')
-            logger.debug('New image %s', new_image)
-            self.config.update({'image_name': new_image})
-        return build_res
-
     def _update_env_vars(self, env):
         """ Export enviroment when create container
             updated the dictionary env_vars
@@ -1476,8 +1465,8 @@ class InstanceV(DockerV):
             if not self.is_running:
                 state = self.exec_cmd('supervisorctl status odoo')
                 res.update({
-                    'result': ('Failed to restart the instance, could not'
-                               ' start odoo: {state}').format(state=state.strip())})
+                    'error': ('Failed to restart the instance, could not'
+                              ' start odoo: {state}').format(state=state.strip())})
                 return res
         except (errors.SupervisorStatusError, errors.CommandError, errors.NotRunning) as error:
             res.update({'error': 'Couldn\'t restart the instance, {error}'.format(
@@ -1559,7 +1548,8 @@ def deactivate_database(config, db_name, extras_sql=False):
                 db.execute(sql)
             except psycopg2.ProgrammingError as error:
                 str_error = str(error)
-                if any([string in str_error for string in ['relation', 'does not exist']]):
+                if any([string in str_error for string in
+                        ['relation', 'does not exist', 'already exists']]):
                     logger.warn('An error ocurred, skipping %s. %s',
                                 name, str_error.split('\n')[0])
                     continue
